/*@ Internet Message Format (RFC 822 -> 2822 -> 5322) parser.
 *
 * Copyright (c) 2024 Steffen Nurpmeso <steffen@sdaoden.eu>.
 * SPDX-License-Identifier: ISC
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef su_IMF_H
#define su_IMF_H
#include <su/code.h>
#if defined su_HAVE_IMF || defined DOXYGEN /*XXX DOXYGEN bug; ifdef su_HAVE_IMF*/
#include <su/mem-bag.h>
#define su_HEADER
#include <su/code-in.h>
C_DECL_BEGIN
struct su_imf_addr;
/* su_imf_addr {{{ */
enum su_imf_mode{
	su_IMF_MODE_NONE,
	su_IMF_MODE_RELAX = 1u<<0,
	su_IMF_MODE_OK_DISPLAY_NAME_DOT = 1u<<1,
	su_IMF_MODE_OK_ADDR_SPEC_NO_DOMAIN = 1u<<2,
	su_IMF_MODE_STOP_EARLY = 1u<<7,
	su__IMF_MODE_MASK = 0xFF
};
enum su_imf_state{
	su_IMF_STATE_RELAX = 1u<<8,
	su_IMF_STATE_DISPLAY_NAME_DOT = 1u<<9,
	su_IMF_STATE_ADDR_SPEC_NO_DOMAIN = 1u<<10,
	su_IMF_STATE_DOMAIN_LITERAL = 1u<<11,
	su_IMF_STATE_GROUP = 1u<<12,
	su_IMF_STATE_GROUP_START = 1u<<13,
	su_IMF_STATE_GROUP_END = 1u<<14,
	su_IMF_STATE_GROUP_EMPTY = 1u<<15,
	su_IMF_STATE_MASK = su_IMF_STATE_DISPLAY_NAME_DOT | su_IMF_STATE_ADDR_SPEC_NO_DOMAIN |
			su_IMF_STATE_DOMAIN_LITERAL | su_IMF_STATE_GROUP | su_IMF_STATE_GROUP_START |
			su_IMF_STATE_GROUP_END | su_IMF_STATE_GROUP_EMPTY
};
enum su_imf_err{
	su_IMF_ERR_RELAX = 1u<<20,
	su_IMF_ERR_GROUP_DISPLAY_NAME_EMPTY = 1u<<21,
	su_IMF_ERR_DISPLAY_NAME_DOT = 1u<<22,
	su_IMF_ERR_ADDR_SPEC = 1u<<23,
	su_IMF_ERR_COMMENT = 1u<<24,
	su_IMF_ERR_DQUOTE = 1u<<25,
	su_IMF_ERR_GROUP_OPEN = 1u<<26,
	su_IMF_ERR_MASK = su_IMF_ERR_GROUP_DISPLAY_NAME_EMPTY | su_IMF_ERR_DISPLAY_NAME_DOT | su_IMF_ERR_ADDR_SPEC |
			su_IMF_ERR_COMMENT | su_IMF_ERR_DQUOTE | su_IMF_ERR_GROUP_OPEN
};
struct su_imf_addr{
	struct su_imf_addr *imfa_next;
	char *imfa_group_display_name;
	char *imfa_display_name;
	char *imfa_locpar;
	char *imfa_domain;
	char *imfa_comm;
	u32 imfa_group_display_name_len;
	u32 imfa_display_name_len;
	u32 imfa_locpar_len;
	u32 imfa_domain_len;
	u32 imfa_comm_len;
	u32 imfa_mse;
	char imfa_dat[VFIELD_SIZE(0)]; /* Storage for any text (single chunk struct) */
};
INLINE void *su_imf_snap_create(struct su_mem_bag *membp){
	ASSERT_RET(membp != NIL, NIL);
#ifdef su_HAVE_MEM_BAG_LOFI
	return su_mem_bag_lofi_snap_create(membp);
#else
	su_mem_bag_auto_snap_create(membp);
	return NIL;
#endif
}
INLINE void su_imf_snap_gut(struct su_mem_bag *membp, void *snap){
	ASSERT_RET_VOID(membp != NIL);
#ifdef su_HAVE_MEM_BAG_LOFI
	su_mem_bag_lofi_snap_gut(membp, snap);
#else
	UNUSED(snap);
	su_mem_bag_auto_snap_gut(membp);
#endif
}
EXPORT s32 su_imf_parse_addr_header(struct su_imf_addr **app, char const *header, BITENUM(u32,su_imf_mode) mode,
		struct su_mem_bag *membp, char const **endptr_or_nil);
/* }}} */
C_DECL_END
#include <su/code-ou.h>
#endif /* su_HAVE_IMF */
#endif /* su_IMF_H */
/* s-itt-mode */
